class Intervals:
    interval_1minute = '1'
    interval_5minutes = '5'
    interval_1hour = '60'
    interval_1day = 'D'
    
class Sectors:
    cfd = 'cfd'
    america = 'america'
    iceland = 'iceland'
    japan = 'japan'
    luxembourg = 'luxembourg'
    uk = 'uk'
    russia = 'russia'
    australia = 'australia'
    bahrain = 'bahrain'
    belgium = 'belgium'
    brazil = 'brazil'
    vietnam = 'vietnam'
    germany = 'germany'
    hongkong = 'hongkong'
    denmark = 'denmark'
    egypt = 'egypt'
    israel = 'israel'
    india = 'india'
    indonesia = 'indonesia'
    spain = 'spain'
    italy = 'italy'
    canada = 'canada'
    qatar = 'qatar'
    china = 'china'
    colombia = 'colombia'
    latvia = 'latvia'
    lithuania = 'lithuania'
    malaysia = 'malaysia'
    mexico = 'mexico'
    nigeria = 'nigeria'
    netherlands = 'netherlands'
    newzealand = 'newzealand'
    norway = 'norway'
    uae = 'uae'
    peru = 'peru'
    poland = 'poland'
    portugal = 'portugal'
    ksa = 'ksa'
    serbia = 'serbia'
    singapore = 'singapore'
    thailand = 'thailand'
    taiwan = 'taiwan'
    turkey = 'turkey'
    finland = 'finland'
    france = 'france'
    chile = 'chile'
    switzerland = 'switzerland'
    sweden = 'sweden'
    estonia = 'estonia'
    rsa = 'rsa'
    korea = 'korea'